from django.views.generic import ListView, TemplateView
from django.db.models import Q
from django.shortcuts import redirect
from django.template.defaultfilters import slugify
from django.shortcuts import render
from django.core.paginator import EmptyPage, PageNotAnInteger, Paginator
from django.http import Http404
from django.db.models import Count

from snowebsvg.settings import SVG_DEFAULT_VARIANT, SVG_DEFAULT_THEME
from snowebsvg.models import Collection, Svg
from app.forms import \
    ThemeDarkForm, \
    ThemeLightForm, \
    ThemeLightAppForm, \
    ThemeDarkAppForm
from app.templatetags.settings import current_settings

MAX_SVG_RESULTS = 275


class SvgListView(ListView):
    model = Svg
    template_name = 'app/svg_detail.html'

    def get_context_data(self, **kwargs):
        context = super(SvgListView, self).get_context_data(**kwargs)
        context['tag_related'] = Collection.objects.all()
        return context

    def get_queryset(self):
        group_key = self.kwargs.get('group_key')
        collection_key = self.kwargs.get('collection_key')
        svg_key = self.kwargs.get('svg_key')
        if group_key and collection_key and svg_key:
            objects = self.model.objects.filter(
                group__key=group_key,
                group__collection__key=collection_key,
                key=svg_key
            )
            if objects.count() == 0:
                raise Http404
            return objects
        return self.model.objects.all()


class SvgSearchView(TemplateView):
    template_name = 'app/svg_list.html'

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        page = request.GET.get('page', 1)
        paginator = Paginator(context['qs'], MAX_SVG_RESULTS)
        try:
            search_results = paginator.page(page)
        except PageNotAnInteger:
            search_results = paginator.page(1)
        except EmptyPage:
            search_results = paginator.page(paginator.num_pages)
        context['object_list'] = search_results
        return self.render_to_response(context)

    def get_queryset(self):
        key = self.kwargs.get('key')
        if key:
            qs = Svg.objects.filter(
                Q(key__icontains=key) |
                Q(group__key__icontains=key) |
                Q(group__collection__key__icontains=key)
            )
        else:
            qs = Svg.objects.all()
        return qs.order_by('group__key')

    def get_context_data(self, **kwargs):
        context = super(SvgSearchView, self).get_context_data(**kwargs)
        key = self.kwargs.get('key')
        qs = self.get_queryset()
        if key:
            context['key_title'] = key.replace('_', ' ').title()
            # qs_group_key = qs.values('group__key').annotate(Count('group__key')).order_by('-group__key__count')
            # context['tag_related'] = [Svg(key=group_key['group__key']) for group_key in qs_group_key]
        else:
            pass
            # context['tag_related'] = Collection.objects.all()
        context['tag_related'] = Collection.objects.all()
        context['key'] = key
        context['qs'] = qs
        return context

    def post(self, request, *args, **kwargs):
        key = request.POST.get('key', None)
        if key:
            key = key.replace(' ', '_')
            key = key.replace('-', '_')
            key_slug = slugify(key)
            if key_slug:
                return redirect('app:svg_search', key=key_slug)
        return redirect('app:svg_search')


class SvgSettingsView(TemplateView):
    template_name = 'app/svg_settings.html'
    forms = (
        ('form_theme_dark', ThemeDarkForm),
        ('form_theme_light', ThemeLightForm),
        ('form_theme_dark_app', ThemeDarkAppForm),
        ('form_theme_light_app', ThemeLightAppForm),
    )

    def get_context_data(self, request, **kwargs):
        context = {}
        for form_key, form in self.forms:
            context[form_key] = form(current_settings(request))
        context['tag_related'] = Collection.objects.all()
        return context

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(request, **kwargs)
        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        form_key = request.POST.get('form_key', None)
        if form_key and any(tuple_form[0] == form_key for tuple_form in self.forms):
            form = dict(self.forms)[form_key](request.POST)
            if form.is_valid():
                for key, value in form.clean().items():
                    request.session['settings'][key] = value
                request.session.modified = True
            else:
                context = self.get_context_data(request, **kwargs)
                context[form_key] = form
                return render(request, self.template_name, context)
        elif request.POST.get('reset', None) == 'reset':
            request.session['settings'] = {}
            request.session['theme'] = SVG_DEFAULT_THEME
            request.session['variant'] = SVG_DEFAULT_VARIANT
        return render(
            request,
            self.template_name,
            self.get_context_data(request, **kwargs)
        )



import logging
from django.test import TestCase
from django.core import management

from snowebsvg.models import Svg, Collection, GroupSvg
from snowebsvg import settings
from snowebsvg.templatetags.svg import \
    svg_inline, \
    svg_django, \
    collection_styles, \
    svg_preview, \
    svg_stylesheets

logger = logging.getLogger('snowebsvg')


class SnowebSvgTest(TestCase):

    def svg_build(self):
        if Svg.objects.all().count() == 0:
            management.call_command('svg_build')

    def test_svg_sync(self):
        self.svg_build()
        management.call_command('svg_sync')
        self.assertGreater(
            Svg.objects.all().count(),
            0
        )
        self.assertGreater(
            GroupSvg.objects.all().count(),
            0
        )
        self.assertGreater(
            Svg.objects.all().count(),
            0
        )

    def test_templatetags_svg_inline(self):
        self.svg_build()
        for svg in Svg.objects.all():
            logger.debug("svg_inline svg=`%s`" % str(svg))
            logger.debug("svg_inline group_svg=`%s`" % str(svg.group))
            svg_inline(svg)
            svg_inline(svg, variant='glass')
        for svg in Svg.objects.all():
            logger.debug("svg_inline key_composer=`%s`" % str(svg.key_composer))
            svg_inline(svg.key_composer)

    def test_templatetags_svg_django(self):
        self.svg_build()
        for svg in Svg.objects.all():
            logger.debug("svg_django svg=`%s`" % str(svg))
            svg_django(svg)
        for svg in Svg.objects.all():
            logger.debug("svg_django key_composer=`%s`" % str(svg.key_composer))
            svg_django(svg.key_composer)

    def test_templatetags_svg_preview(self):
        self.svg_build()
        for svg in Svg.objects.all():
            logger.debug("svg_preview svg=`%s`" % str(svg))
            svg_preview(svg)
        for svg in Svg.objects.all():
            logger.debug("svg_preview key_composer=`%s`" % str(svg.key_composer))
            svg_preview(svg.key_composer)

    def test_templatetags_collection_styles(self):
        self.svg_build()
        for collection in Collection.objects.all():
            logger.debug("collection_styles collection=`%s`" % str(collection))
            collection_styles(collection)

    def test_svg_stylesheets(self):
        bundles = svg_stylesheets('themes')
        self.assertEqual(
            bundles,
            """<link rel="stylesheet" href="%sthemes-%s.css">""" % (
                settings.BASE_URL_CSS,
                settings.VERSION
            )
        )

    def test_key_decomposer_not_found(self):
        svg_inline('this_key_does_not_exist')



from datetime import datetime
from uuid import uuid4

from django import template
from django.template.loader import get_template
from django.template.exceptions import TemplateDoesNotExist

register = template.Library()


@register.simple_tag
def scale(scale_x, scale_y=None, x=50, y=50):
    if not scale_y:
        scale_y = scale_x
    x_calc = (1 - scale_x) * x
    y_calc = (1 - scale_y) * y
    return "translate({:.2f}, {:.2f}) scale({:.2f}, {:.2f})".format(
        x_calc,
        y_calc,
        scale_x,
        scale_y
    )


@register.simple_tag
def parse(list_props):
    return [p.split(';') for p in list_props.split(' ')]


@register.filter
def to_int(value):
    return int(value)


@register.simple_tag
def variant_manager(variant=None, variant_part='defs'):
    if variant:
        try:
            svg_template = get_template("snowebsvg/common/variants/%s/%s.html" % (
                variant,
                variant_part
            ))
            return svg_template.render()
        except TemplateDoesNotExist:
            return ''
    return ''


@register.simple_tag
def unique_id():
    return datetime.now().strftime('%Y%m-%d%H-%M%S-') + str(uuid4())



from django import template
from snowebsvg.models import Svg
from snowebsvg import settings as svg_settings
from django.conf import settings

register = template.Library()

SVG_DEFAULT_THEME = getattr(settings, 'SVG_DEFAULT_THEME', None)
SVG_DEFAULT_WIDTH = getattr(settings, 'SVG_DEFAULT_WIDTH', None)
SVG_DEFAULT_HEIGHT = getattr(settings, 'SVG_DEFAULT_HEIGHT', None)
SVG_DEFAULT_VARIANT = getattr(settings, 'SVG_DEFAULT_VARIANT', None)


@register.simple_tag
def svg_inline(svg_target, theme=SVG_DEFAULT_THEME, width=SVG_DEFAULT_WIDTH, height=SVG_DEFAULT_HEIGHT,
               variant=SVG_DEFAULT_VARIANT, grid=False, klass=None):
    if isinstance(svg_target, Svg):
        return svg_target.render_html(theme, width, height, variant, grid, klass)
    else:
        svg = Svg()
        svg_composed = svg.key_decomposer(svg_target)
        return svg_composed.render_html(theme, width, height, variant, grid, klass)


@register.simple_tag
def svg_preview(svg_target, theme=SVG_DEFAULT_THEME, width=SVG_DEFAULT_WIDTH, height=SVG_DEFAULT_HEIGHT,
                variant=SVG_DEFAULT_VARIANT, grid=False):
    if isinstance(svg_target, Svg):
        return svg_target.render_preview(theme, width, height, variant, grid)
    else:
        svg = Svg()
        svg_composed = svg.key_decomposer(svg_target)
        return svg_composed.render_preview(theme, width, height, variant, grid)


@register.simple_tag
def svg_django(svg_target, theme=SVG_DEFAULT_THEME, width=SVG_DEFAULT_WIDTH, height=SVG_DEFAULT_HEIGHT,
               variant=SVG_DEFAULT_VARIANT):
    if isinstance(svg_target, Svg):
        return svg_target.render_django(theme, width, height, variant)
    else:
        svg = Svg()
        svg_composed = svg.key_decomposer(svg_target)
        return svg_composed.render_django(theme, width, height, variant)


@register.simple_tag
def collection_styles(collection):
    return collection.render_styles()


@register.simple_tag
def svg_stylesheets(*bundles):
    ret = ""
    for bundle in bundles:
        ret += ("<link rel=\"stylesheet\" href=\"%s%s-%s.css\">" % (
            svg_settings.BASE_URL_CSS,
            bundle,
            svg_settings.VERSION
        ))
    return ret



import re
from django.forms import Form
from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

regex_hexadecimal = re.compile("^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$")


def is_valid_hexadecimal(value):
    is_hexadecimal = re.search(regex_hexadecimal, value)
    if not is_hexadecimal:
        raise ValidationError(
            _('%(value)s is not a hexadecimal color'),
            params={'value': value},
        )
    return True


def is_valid_background_color(value):
    try:
        is_valid_hexadecimal(value)
    except ValidationError:
        if not value.startswith('radial-gradient(') and not value.startswith('linear-gradient('):
            raise ValidationError(
                _('%(value)s is not a valid css property'),
                params={'value': value},
            )


class HexFormField(forms.CharField):
    default_validators = [is_valid_hexadecimal]


class BackgroundColorFormField(forms.CharField):
    default_validators = [is_valid_background_color]
    max_length = 200


# Snoweb SVG

class ThemeLightForm(Form):
    theme_light_primary = HexFormField(
        required=False,
        label=_('Color theme light primary'),
        help_text=_('Hexadecimal color'),
        widget=forms.TextInput(attrs={'class': 'bg-light-0 dark:bg-dark-0 text-light-900 dark:text-dark-900'})
    )
    theme_light_secondary = HexFormField(
        required=False,
        label=_('Color theme light secondary'),
        help_text=_('Hexadecimal color'),
        widget=forms.TextInput(attrs={'class': 'bg-light-0 dark:bg-dark-0 text-light-900 dark:text-dark-900'})
    )
    theme_light_tertiary = HexFormField(
        required=False,
        label=_('Color theme light tertiary'),
        help_text=_('Hexadecimal color'),
        widget=forms.TextInput(attrs={'class': 'bg-light-0 dark:bg-dark-0 text-light-900 dark:text-dark-900'})
    )


class ThemeDarkForm(Form):
    theme_dark_primary = HexFormField(
        required=False,
        label=_('Color theme dark primary'),
        help_text=_('Hexadecimal color'),
        widget=forms.TextInput(attrs={'class': 'bg-light-0 dark:bg-dark-0 text-light-900 dark:text-dark-900'})
    )
    theme_dark_secondary = HexFormField(
        required=False,
        label=_('Color theme dark secondary'),
        help_text=_('Hexadecimal color'),
        widget=forms.TextInput(attrs={'class': 'bg-light-0 dark:bg-dark-0 text-light-900 dark:text-dark-900'})
    )
    theme_dark_tertiary = HexFormField(
        required=False,
        label=_('Color theme dark tertiary'),
        help_text=_('Hexadecimal color'),
        widget=forms.TextInput(attrs={'class': 'bg-light-0 dark:bg-dark-0 text-light-900 dark:text-dark-900'})
    )


# App

class ThemeLightAppForm(Form):
    theme_light_background_body = BackgroundColorFormField(
        required=False,
        label=_('Color theme light background body'),
        help_text=_('Hexadecimal color, linear-gradient, radial-gradient'),
        widget=forms.TextInput(attrs={'class': 'bg-light-0 dark:bg-dark-0 text-light-900 dark:text-dark-900'})
    )


class ThemeDarkAppForm(Form):
    theme_dark_background_body = BackgroundColorFormField(
        required=False,
        label=_('Color theme dark background body'),
        help_text=_('Hexadecimal color, linear-gradient, radial-gradient'),
        widget=forms.TextInput(attrs={'class': 'bg-light-0 dark:bg-dark-0 text-light-900 dark:text-dark-900'})
    )



import sys
import logging
from django.core.management.color import color_style


class DjangoColorsFormatter(logging.Formatter):
    def __init__(self, *args, **kwargs):
        super(DjangoColorsFormatter, self).__init__(*args, **kwargs)
        self.style = self.configure_style(color_style())

    def configure_style(self, style):
        style.DEBUG = style.HTTP_NOT_MODIFIED
        style.INFO = style.HTTP_INFO
        style.WARNING = style.HTTP_NOT_FOUND
        style.ERROR = style.ERROR
        style.CRITICAL = style.HTTP_SERVER_ERROR
        return style

    def format(self, record):
        message = logging.Formatter.format(self, record)
        if sys.version_info[0] < 3:
            if isinstance(message, str):
                message = message.encode('utf-8')
        colorizer = getattr(self.style, record.levelname, self.style.HTTP_SUCCESS)
        return colorizer(message)



import os
import json
import pkg_resources

BASE_URL_CSS = "https://static.snoweb.io/snowebsvg/dist/css/"

try:
    #  Production part
    VERSION = pkg_resources.get_distribution("snowebsvg").version
except pkg_resources.DistributionNotFound:
    #  Develop part
    BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    with open(os.path.join(BASE_DIR, 'package.json')) as package:
        data = json.load(package)
        VERSION = data['version']

SVG_DEFAULT_THEME = 'light'
SVG_DEFAULT_WIDTH = 100
SVG_DEFAULT_HEIGHT = 100
SVG_DEFAULT_VARIANT = None
SVG_DEFAULT_KEY = 'decorator-rect-basic'

try:
    import wagtail
    WAGTAIL_INSTALL = True
except ImportError:
    WAGTAIL_INSTALL = False



import logging
from decouple import config
from time import time, sleep
import psycopg2
import requests
from django.core import management
from django.core.cache import cache
from django.conf import settings
from snowebsvg import settings as snowebsvg_settings

check_timeout = 30
check_interval = 1
interval_unit = "second" if check_interval == 1 else "seconds"
start_time = time()
logger = logging.getLogger()
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler())
assert check_timeout > 0
assert check_interval < check_timeout


def pg_is_ready(host, user, password, dbname, port):
    while time() - start_time < check_timeout:
        try:
            conn = psycopg2.connect(**vars())
            logger.info("Postgres is ready")
            conn.close()
            return True
        except psycopg2.OperationalError:
            logger.info(f"Postgres isn't ready. Waiting for {check_interval} {interval_unit}...")
            sleep(check_interval)

    logger.error(f"We could not connect to Postgres within {check_timeout} seconds.")
    return False


class Command(management.base.BaseCommand):

    def handle(self, *args, **options):
        # Wait for postgresql
        if not settings.DEBUG:
            pg_is_ready(
                'snowebsvg_db',
                config('DATABASE_USER', default='postgres'),
                config('DATABASE_PASSWORD', default='postgres'),
                config('DATABASE_NAME', default='postgres'),
                '5432'
            )
            if settings.AWS_SECRET_ACCESS_KEY and settings.AWS_ACCESS_KEY_ID:
                self.stdout.write("Step collect static")
                management.call_command('collectstatic', interactive=False)

        self.stdout.write("Step migrate")
        management.call_command('migrate')

        self.stdout.write("Step svg build")
        management.call_command('svg_build')

        self.stdout.write("Step compile messages")
        management.call_command('compilemessages')

        self.stdout.write("Step clear cache")
        cache.clear()



import os
from django.db import models
from django.utils.translation import gettext_lazy as _
from django.template.loader import get_template
from django.utils.functional import cached_property
from django.template.loaders.app_directories import Loader
from django.template.exceptions import TemplateDoesNotExist
from snowebsvg import settings


class Collection(models.Model):
    key = models.CharField(max_length=255, verbose_name=_("Key"))

    class Meta:
        ordering = ('key',)

    def __str__(self):
        return self.key.title()

    @cached_property
    def root_directory(self):
        for template_directory in Loader('django').get_dirs():
            template_directory = str(template_directory)
            if '/snowebsvg/' in template_directory:
                base_dir_svg = "%s/snowebsvg" % template_directory
                return "%s/%s" % (
                    base_dir_svg,
                    'collections'
                )

    def build(self):
        """
        Build all collection group's available in django apps template folder named :
        ``snowebsvg/collections/<collection_key>``
        """
        conf_files = [
            'django.html',
            'styles.html',
            '_preview.html',
        ]
        for group_key in os.listdir("%s/%s" % (self.root_directory, self.key)):
            if group_key not in conf_files:
                group, _ = GroupSvg.objects.get_or_create(key=group_key, collection_id=self.id)
                group.build()

    def render_styles(self):
        """
        Method for rendering stylesheet code
        """

        context = {
            'self': self,
            'settings': settings
        }
        try:
            svg_template = get_template("snowebsvg/collections/%s/styles.html" % self.key)
            return svg_template.render(context)
        except TemplateDoesNotExist:
            svg_template = get_template("snowebsvg/common/styles.html")
            return svg_template.render(context)


class GroupSvg(models.Model):
    key = models.CharField(max_length=255, verbose_name=_("Key"))
    collection = models.ForeignKey(
        Collection,
        default=None,
        on_delete=models.CASCADE,
        null=True
    )

    class Meta:
        ordering = ('key',)

    def __str__(self):
        return self.key.title()

    @cached_property
    def path_entry(self):
        """
        File path of a ``GroupSvg``
        """
        return "%s/%s/%s" % (
            self.collection.root_directory,
            self.collection.key,
            self.key
        )

    def build(self):
        """
        Build all ``Svg`` available in django apps template folder named :
        ``snowebsvg/collections/<collection_key>/<group_svg_key>``
        """
        try:
            for html_filename in os.listdir(self.path_entry):
                svg_key = html_filename.replace('.html', '')
                # We don't build private files
                if not svg_key.startswith('_'):
                    try:
                        Svg.objects.get(
                            key=svg_key,
                            group_id=self.id
                        )
                    except Svg.DoesNotExist:
                        svg = Svg(
                            key=svg_key,
                            group_id=self.id
                        )
                        svg.save()
        except NotADirectoryError:
            # `self.path_entry` is not a directory, pass it
            pass


if settings.WAGTAIL_INSTALL:
    from wagtail.search import index

    class AbstractSvg(index.Indexed, models.Model):
        class Meta:
            abstract = True
else:
    class AbstractSvg(models.Model):
        class Meta:
            abstract = True


class Svg(AbstractSvg):
    key = models.CharField(max_length=255, verbose_name=_("Key"))
    group = models.ForeignKey(
        GroupSvg,
        default=None,
        on_delete=models.CASCADE,
        null=True
    )

    if settings.WAGTAIL_INSTALL:
        search_fields = [
            index.SearchField('key_composer', partial_match=True),
        ]

    class Meta:
        ordering = ('key',)

    def __str__(self):
        return self.key_composer.title().replace('-', ' ').replace('_', ' ')

    @cached_property
    def path_entry(self):
        """
        :return string: Path html of Svg
        """
        return "snowebsvg/collections/%s/%s/%s.html" % (
            self.group.collection.key,
            self.group.key,
            self.key
        )

    @cached_property
    def key_composer(self):
        """
        :return string: Unique Identifier of Svg
        """
        return "%s-%s-%s" % (
            self.group.collection.key if self.group and self.group.collection else '',
            self.group.key if self.group else '',
            self.key
        )

    @cached_property
    def default_svg(self):
        collection_key, group_key, key = settings.SVG_DEFAULT_KEY.split('-')
        collection, _ = Collection.objects.get_or_create(
            key=collection_key
        )
        group, _ = GroupSvg.objects.get_or_create(
            key=group_key,
            collection_id=collection.id
        )
        svg, _ = Svg.objects.get_or_create(
            key=key,
            group_id=group.id
        )
        return svg

    def key_decomposer(self, key_composer):
        """
        :return svg: Svg instance by key composer
        """
        try:
            collection_key, group_key, key = key_composer.split('-')
            return Svg.objects.get(
                key=key,
                group__key=group_key,
                group__collection__key=collection_key
            )
        except (Svg.DoesNotExist, ValueError):
            return self.default_svg

    def render_django(self, theme=settings.SVG_DEFAULT_THEME, width=settings.SVG_DEFAULT_WIDTH,
                      height=settings.SVG_DEFAULT_HEIGHT, variant=settings.SVG_DEFAULT_VARIANT):
        """
        Method for rendering Django template code

        :param theme: Theme, defaults to :ref:`SVG_DEFAULT_THEME <references_settings>`
        :param width: Width, defaults :ref:`SVG_DEFAULT_WIDTH <references_settings>`
        :param height: Height, defaults :ref:`SVG_DEFAULT_HEIGHT <references_settings>`
        :param variant: Variant, defaults :ref:`SVG_DEFAULT_VARIANT <references_settings>`
        """
        context = {
            'self': self,
            'theme': theme,
            'width': width,
            'height': height,
            'variant': variant,
            'start': '{%',
            'end': '%}'
        }
        try:
            svg_template = get_template("snowebsvg/collections/%s/%s/_django.html" % (
                self.group.collection.key,
                self.group.key,
            ))
            return svg_template.render(context)
        except TemplateDoesNotExist:
            svg_template = get_template("snowebsvg/common/django.html")
            return svg_template.render(context)

    def render_html(self, theme=settings.SVG_DEFAULT_THEME, width=settings.SVG_DEFAULT_WIDTH,
                    height=settings.SVG_DEFAULT_HEIGHT, variant=settings.SVG_DEFAULT_VARIANT,
                    grid=False, klass=None):
        """
        Method for rendering Svg to HTML

        :param theme: Theme, defaults to :ref:`SVG_DEFAULT_THEME <references_settings>`
        :param width: Width, defaults :ref:`SVG_DEFAULT_WIDTH <references_settings>`
        :param height: Height, defaults :ref:`SVG_DEFAULT_HEIGHT <references_settings>`
        :param grid: Grid, add grid for debugging SVG, defaults to False`
        :param variant: Variant, defaults :ref:`SVG_DEFAULT_VARIANT <references_settings>`
        :param klass: extra css class
        """
        try:
            svg = self
            svg_template = get_template(self.path_entry)
        except TemplateDoesNotExist:
            svg = self.default_svg
            svg_template = get_template(svg.path_entry)
        return svg_template.render({
            'self': svg,
            'theme': theme,
            'width': width,
            'height': height,
            'grid': grid,
            'variant': variant,
            'klass': klass
        })

    def render_preview(self, theme=settings.SVG_DEFAULT_THEME, width=settings.SVG_DEFAULT_WIDTH,
                       height=settings.SVG_DEFAULT_HEIGHT, variant=settings.SVG_DEFAULT_VARIANT, grid=False):
        """
        Method for rendering a preview of Svg to HTML

        If _preview.html exist, then render the preview html file.
        If _preview.html does not exist, then fall back to the common preview.html file.

        :param theme: Theme, defaults to :ref:`SVG_DEFAULT_THEME <references_settings>`
        :param width: Width, defaults :ref:`SVG_DEFAULT_WIDTH <references_settings>`
        :param height: Height, defaults :ref:`SVG_DEFAULT_HEIGHT <references_settings>`
        :param grid: Grid, add grid for debugging SVG, defaults to False`
        :param variant: Variant, defaults :ref:`SVG_DEFAULT_VARIANT <references_settings>`

        """
        context = {
            'self': self,
            'theme': theme,
            'width': width,
            'height': height,
            'grid': grid,
            'variant': variant
        }
        try:
            svg_template = get_template("snowebsvg/collections/%s/%s/_preview.html" % (
                self.group.collection.key,
                self.group.key,
            ))
            return svg_template.render(context)
        except TemplateDoesNotExist:
            svg_template = get_template("snowebsvg/common/preview.html")
            return svg_template.render(context)



from django.contrib.sitemaps.views import sitemap
from django.urls import path
from django.views.generic import RedirectView
from django.urls import reverse_lazy
from app.views import SvgListView, SvgSearchView, SvgSettingsView
from app.sitemaps import StaticSitemap, CollectionSitemap, SvgSitemap, GroupSvgSitemap

app_name = 'app'

sitemaps = {
    'static': StaticSitemap,
    'collection': CollectionSitemap,
    'group': GroupSvgSitemap,
    'svg': SvgSitemap,
}

urlpatterns = [
    path('', RedirectView.as_view(permanent=True, url=reverse_lazy('app:svg_search')), name='home'),
    path('svg/settings/', SvgSettingsView.as_view(), name='svg_settings'),
    path('svg/', SvgSearchView.as_view(), name='svg_search'),
    path('svg/<key>/', SvgSearchView.as_view(), name='svg_search'),
    path('svg/<collection_key>/<group_key>/<svg_key>/', SvgListView.as_view(), name='svg'),
    path('svg/sitemap.xml', sitemap, {'sitemaps': sitemaps}, name='django.contrib.sitemaps.views.sitemap')
]



from snowebsvg import settings


def clean_middleware(value):
    if len(value) > 10:
        return ValueError
    if value == 'none':
        value = ''
    return value


conf = (
    ('theme', settings.SVG_DEFAULT_THEME),
    ('variant', settings.SVG_DEFAULT_VARIANT),
)


class SettingsMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if 'settings' not in request.session:
            request.session['settings'] = {}

        for conf_key, conf_default in conf:
            if conf_key not in request.session:
                request.session[conf_key] = conf_default
            if conf_key in request.GET:
                try:
                    value = clean_middleware(request.GET[conf_key])
                    request.session[conf_key] = value
                except ValueError:
                    pass
        return self.get_response(request)



import os
from django.core.management.base import BaseCommand
from snowebsvg.models import Collection, GroupSvg, Svg


class Command(BaseCommand):

    def handle(self, *args, **options):
        for collection in Collection.objects.all():
            collection_path = "%s/%s" % (collection.root_directory, collection.key)
            if not os.path.isdir(collection_path):
                collection.delete()

        for group_svg in GroupSvg.objects.all():
            if not os.path.isdir(group_svg.path_entry):
                group_svg.delete()

        for svg in Svg.objects.all():
            svg_path = "%s/%s.html" % (svg.group.path_entry, svg.key)
            if not os.path.isfile(svg_path):
                svg.delete()



from django.contrib import admin

from snowebsvg.models import Collection, GroupSvg, Svg


class GroupSvgInline(admin.TabularInline):
    model = GroupSvg
    extra = 1


class SvgInline(admin.TabularInline):
    model = Svg
    extra = 1


class CollectionAdmin(admin.ModelAdmin):
    inlines = [
        GroupSvgInline
    ]


class GroupSvgAdmin(admin.ModelAdmin):
    inlines = [
        SvgInline
    ]


admin.site.register(Collection, CollectionAdmin)
admin.site.register(GroupSvg, GroupSvgAdmin)
admin.site.register(Svg)



import os
from django.core.management.base import BaseCommand
from snowebsvg.models import Collection


class Command(BaseCommand):

    def handle(self, *args, **options):
        instance = Collection()
        for collection_name in os.listdir(instance.root_directory):
            # We don't build private files
            if not collection_name.startswith('_'):
                collection, _ = Collection.objects.get_or_create(key=collection_name)
                collection.build()



#!/usr/bin/env python
import os
from setuptools import find_packages, setup
import json

PROJECT_DIR = os.path.dirname(__file__)

with open("README.rst", "r") as fh:
    long_description = fh.read()

with open('./package.json') as package:
    data = json.load(package)
    version = data['version']

setup(
    name='snowebsvg',
    version=version,
    url='https://github.com/Aleksi44/snoweb-svg',
    author="Alexis Le Baron",
    author_email="hello@snoweb.io",
    description="Snoweb SVG",
    long_description=long_description,
    keywords="svg django snoweb",
    license='GPL-3.0',
    install_requires=[],
    platforms=['linux'],
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Operating System :: OS Independent',
        'Topic :: Software Development',
        'Framework :: Wagtail',
        'Framework :: Wagtail :: 2',
        'Framework :: Django',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Framework :: Django :: 3.2',
        'Framework :: Django :: 4.0',
    ]
)



from django.contrib import sitemaps
from django.urls import reverse
from django.contrib.sites.models import Site

from snowebsvg.models import Collection, GroupSvg, Svg


class BaseSitemap(sitemaps.Sitemap):
    priority = 0.4
    protocol = 'https'
    changefreq = 'daily'

    def get_urls(self, site=None, **kwargs):
        site = Site(domain='www.snoweb-svg.com', name='www.snoweb-svg.com')
        return super(BaseSitemap, self).get_urls(site=site, **kwargs)


class CollectionSitemap(BaseSitemap):

    def items(self):
        return Collection.objects.all()

    def location(self, item):
        return reverse('app:svg_search', kwargs={'key': item.key})


class GroupSvgSitemap(BaseSitemap):

    def items(self):
        return GroupSvg.objects.all()

    def location(self, item):
        return reverse('app:svg_search', kwargs={'key': item.key})


class SvgSitemap(BaseSitemap):

    def items(self):
        return Svg.objects.all()

    def location(self, item):
        return reverse('app:svg', kwargs={
            'collection_key': item.group.collection.key,
            'group_key': item.group.key,
            'svg_key': item.key
        })


class StaticSitemap(BaseSitemap):

    def items(self):
        return ['app:svg_settings']

    def location(self, item):
        return reverse(item)



# Generated by Django 3.1.4 on 2021-02-08 11:18

from django.db import migrations, models
import django.db.models.deletion
from snowebsvg import settings

if settings.WAGTAIL_INSTALL:
    import wagtail.search.index
    bases_svg = (wagtail.search.index.Indexed, models.Model)
else:
    bases_svg = (models.Model,)


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Collection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(max_length=255, verbose_name='Key')),
            ],
            options={
                'ordering': ('key',),
            },
        ),
        migrations.CreateModel(
            name='GroupSvg',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(max_length=255, verbose_name='Key')),
                ('collection', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='snowebsvg.collection')),
            ],
            options={
                'ordering': ('key',),
            },
        ),
        migrations.CreateModel(
            name='Svg',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(max_length=255, verbose_name='Key')),
                ('group', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='snowebsvg.groupsvg')),
            ],
            options={
                'ordering': ('key',),
            },
            bases=bases_svg,
        ),
    ]
